﻿using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Problems.DataAnalysis;
using HeuristicLab.Problems.DataAnalysis.Transformations;
using HeuristicLab.DataPreprocessing.Interfaces;

namespace HeuristicLab.DataPreprocessing.Implementations
{
  public class FilteredPreprocessingData : NamedItem, IFilteredPreprocessingData
  {
    protected ITransactionalPreprocessingData originalData;
    protected ITransactionalPreprocessingData filteredData;

    protected FilteredPreprocessingData(FilteredPreprocessingData original, Cloner cloner)
      : base(original, cloner) {
        originalData = (ITransactionalPreprocessingData)original.originalData;
        filteredData = (ITransactionalPreprocessingData)original.filteredData;
    }

    public FilteredPreprocessingData(ITransactionalPreprocessingData preporcessingData)
      : base() {
        originalData = preporcessingData;
        filteredData = null;
    }

     public override IDeepCloneable Clone(Cloner cloner)
     {
       return new FilteredPreprocessingData(this, cloner);
     }

     public T GetCell<T>(int columnIndex, int rowIndex)
     {
       return ActiveData.GetCell<T>(columnIndex, rowIndex);
     }

     public void SetCell<T>(int columnIndex, int rowIndex, T value)
     {
       ReadOnlyOnFilterData.SetCell<T>(columnIndex, rowIndex, value);
     }

     public string GetCellAsString(int columnIndex, int rowIndex)
     {
       return ActiveData.GetCellAsString(columnIndex, rowIndex);
     }

     public IList<T> GetValues<T>(string variableName)
     {
       return ActiveData.GetValues<T>(variableName);
     }

     public IList<T> GetValues<T>(int columnIndex)
     {
       return ActiveData.GetValues<T>(columnIndex);
     }

     public void SetValues<T>(int columnIndex, IList<T> values)
     {
       ReadOnlyOnFilterData.SetValues<T>(columnIndex, values);
     }

     public void InsertRow(int rowIndex)
     {
       ReadOnlyOnFilterData.InsertRow(rowIndex);
     }

     public void DeleteRow(int rowIndex)
     {
       ReadOnlyOnFilterData.DeleteRow(rowIndex);
     }

     public void InsertColumn<T>(string variableName, int columnIndex)
     {
       ReadOnlyOnFilterData.InsertColumn<T>(variableName, columnIndex);
     }

     public void DeleteColumn(int columnIndex)
     {
       ReadOnlyOnFilterData.DeleteColumn(columnIndex);
     }

     public IntRange TrainingPartition
     {
       get { return originalData.TrainingPartition; }
     }

     public IntRange TestPartition
     {
       get { return originalData.TestPartition; }
     }

     public IList<ITransformation> Transformations
     {
       get { return originalData.Transformations; }
     }

     public IEnumerable<string> VariableNames
     {
       get { return ActiveData.VariableNames; }
     }

     public string GetVariableName(int columnIndex)
     {
       return ActiveData.GetVariableName(columnIndex);
     }

     public int GetColumnIndex(string variableName)
     {
       return ActiveData.GetColumnIndex(variableName);
     }

     public bool IsType<T>(int columnIndex)
     {
       return originalData.IsType<T>(columnIndex);
     }

     public int Columns
     {
       get { return ActiveData.Columns; }
     }

     public int Rows
     {
       get { return ActiveData.Rows; }
     }

     public Dataset ExportToDataset()
     {
       return originalData.ExportToDataset();
     }

     public void SetFilter(bool[] rowFilters) {

       filteredData = (ITransactionalPreprocessingData)originalData.Clone();

       for (int row = (rowFilters.Length - 1); row >= 0; --row)
       {
         if (rowFilters[row])
         {
           filteredData.DeleteRow(row);
         }
       }
     }

     public void PersistFilter() {
       originalData = (ITransactionalPreprocessingData)filteredData.Clone();
       ResetFilter();
     }

     public void ResetFilter() {
       filteredData = null;
     }

     public ITransactionalPreprocessingData ActiveData
     {
       get { return IsFiltered ? filteredData : originalData; }
     }

     public ITransactionalPreprocessingData ReadOnlyOnFilterData
     {
       get {
         if (IsFiltered)
            throw new InvalidOperationException();
         
         return originalData;
       }
     }

     public bool IsFiltered {
       get { return filteredData != null; }
     }

     public event DataPreprocessingChangedEventHandler Changed;

     public bool IsUndoAvailable
     {
       get { return IsFiltered ? false : originalData.IsUndoAvailable; }
     }

     public void Undo()
     {
       ReadOnlyOnFilterData.Undo();
     }

     public void InTransaction(Action action, DataPreprocessingChangedEventType type = DataPreprocessingChangedEventType.Any)
     {
       ReadOnlyOnFilterData.InTransaction(action, type);
     }

     public void BeginTransaction(DataPreprocessingChangedEventType type)
     {
       ReadOnlyOnFilterData.BeginTransaction(type);
     }

     public void EndTransaction()
     {
       originalData.EndTransaction();
     }
  }
}
